import SWaN_accel.spectrum as spectrum
import SWaN_accel.orientation as orientation
import pandas as pd
import SWaN_accel.config as config
import SWaN_accel.utils as utils
import SWaN_accel.energy as energy
import SWaN_accel.signal_magnitude as signal_magnitude
from numpy.linalg import norm
import numpy as np

def compute_extra_features(X,sampling):
   
    feature_list = []

    spectrum_feature_extractor = spectrum.FrequencyFeature(X, sr=sampling)
    spectrum_feature_extractor.fft()
    spectrum_feature_extractor.peaks()
    feature_list.append(spectrum_feature_extractor.dominant_frequency())
    feature_list.append(spectrum_feature_extractor.dominant_frequency_power())
    feature_list.append(spectrum_feature_extractor.total_power())


    ori_feature_extractor = orientation.OrientationFeature(X, subwins=config.winSize)
    ori_feature_extractor.estimate_orientation(unit='deg')
    feature_list.append(ori_feature_extractor.ori_x_median())
    feature_list.append(ori_feature_extractor.ori_y_median())
    feature_list.append(ori_feature_extractor.ori_z_median())
    feature_list.append(ori_feature_extractor.ori_var_sum())
    feature_list.append(ori_feature_extractor.ori_range_max())
    feature_list.append(ori_feature_extractor.range_x_angle())
    feature_list.append(ori_feature_extractor.range_y_angle())
    feature_list.append(ori_feature_extractor.range_z_angle())

    feature_list.append(ori_feature_extractor.zerocross_x_angle())
    feature_list.append(ori_feature_extractor.zerocross_y_angle())
    feature_list.append(ori_feature_extractor.zerocross_z_angle())

    X_vm = utils.vec2colarr(norm(X, ord=2, axis=1))
    energy_feature_extractor = energy.EnergyFeature(X_vm, subwins=30)
    energy_feature_extractor.get_energies()
    feature_list.append(energy_feature_extractor.smv_energy_sum())
    feature_list.append(energy_feature_extractor.smv_energy_var())

    X_vm_norm = utils.vec2colarr(norm(X, ord=2, axis=1))-1
    vm_norm_feature_extractor = signal_magnitude.EnergyFeature(X_vm, subwins=30)
    vm_norm_feature_extractor.get_energies()
    feature_list.append(vm_norm_feature_extractor.smv_energy_sum())
    feature_list.append(vm_norm_feature_extractor.smv_energy_var())




    ## calculate autocorrelation (create a dataframe for each feature)


    # def estimated_autocorrelation(x):
    #     """
    #     http://stackoverflow.com/q/14297012/190597
    #     http://en.wikipedia.org/wiki/Autocorrelation#Estimation
    #     """
    #     n = len(x)
    #     variance = x.var()
    #     x = x - x.mean()
    #     r = np.correlate(x, x, mode='full')[-n:]
    #     assert np.allclose(r, np.array([(x[:n - k] * x[-(n - k):]).sum() for k in range(n)]))
    #     result = r / (variance * (np.arange(n, 0, -1)))
    #     return result




    result = pd.concat(feature_list, axis=1)
    return result



"""

Computing features about accelerometer orientations

Author: Qu Tang

Date: Jul 10, 2018
"""
import numpy as np
from numpy.linalg import norm
from SWaN_accel.utils import *
from math import *

class OrientationFeature:
    def __init__(self, X, subwins=4):
        self._X = X
        self._subwins = subwins

    @staticmethod
    def orientation_xyz(X, unit='rad'):
        X = as_float64(X)
        if not has_enough_samples(X):
            print(
                '''One of sub windows do not have enough samples, will ignore in
                feature computation''')
            orientation_xyz = np.array([np.nan, np.nan, np.nan])
        else:


            mdn_val = np.median(X, axis=0)
            x_mdn, y_mdn, z_mdn = mdn_val[0], mdn_val[1], mdn_val[2]

            denom = np.sqrt(x_mdn * x_mdn + y_mdn * y_mdn + z_mdn * z_mdn)
            x_mdn_norm, y_mdn_norm, z_mdn_norm = x_mdn / denom, y_mdn / denom, z_mdn / denom

            x_ang = acos(x_mdn_norm) * (180 / pi)
            y_ang = acos(y_mdn_norm) * (180 / pi)
            z_ang = acos(z_mdn_norm) * (180 / pi)
            orientation_xyz = np.array([x_ang, y_ang, z_ang])

        return vec2rowarr(orientation_xyz)

    def estimate_orientation(self, unit='rad'):
        result = apply_over_subwins(
            self._X, OrientationFeature.orientation_xyz, subwins=self._subwins, unit=unit)
        self._orientations = np.concatenate(result, axis=0)
        return self

    def median_angles(self):
        median_angles = np.nanmedian(self._orientations, axis=0)
        result = vec2rowarr(np.array(median_angles))
        result = add_name(result, self.median_angles.__name__)
        return result

    def median_x_angle(self):
        median_angles = np.nanmedian(self._orientations, axis=0)
        result = vec2rowarr(np.array([median_angles[0]]))
        result = add_name(result, self.median_x_angle.__name__)
        return result

    def ori_x_median(self):
        median_angles = np.nanmedian(self._orientations, axis=0)
        result = vec2rowarr(np.array([median_angles[0]]))
        result = add_name(result, self.ori_x_median.__name__)
        return result

    def ori_y_median(self):
        median_angles = np.nanmedian(self._orientations, axis=0)
        result = vec2rowarr(np.array([median_angles[1]]))
        result = add_name(result, self.ori_y_median.__name__)
        return result

    def ori_z_median(self):
        median_angles = np.nanmedian(self._orientations, axis=0)
        result = vec2rowarr(np.array([median_angles[2]]))
        result = add_name(result, self.ori_z_median.__name__)
        return result

    def median_y_angle(self):
        median_angles = np.nanmedian(self._orientations, axis=0)
        result = vec2rowarr(np.array([median_angles[1]]))
        result = add_name(result, self.median_y_angle.__name__)
        return result

    def median_z_angle(self):
        median_angles = np.nanmedian(self._orientations, axis=0)
        result = vec2rowarr(np.array([median_angles[2]]))
        result = add_name(result, self.median_z_angle.__name__)
        return result

    def range_angles(self):
        range_angles = abs(np.nanmax(
            self._orientations, axis=0) - np.nanmin(self._orientations, axis=0))
        result = vec2rowarr(np.array(range_angles))
        result = add_name(result, self.range_angles.__name__)
        return result

    def range_x_angle(self):
        range_angles = np.nanmax(
            self._orientations, axis=0) - np.nanmin(self._orientations, axis=0)
        result = vec2rowarr(np.array([range_angles[0]]))
        result = add_name(result, self.range_x_angle.__name__)
        return result

    def range_y_angle(self):
        range_angles = np.nanmax(
            self._orientations, axis=0) - np.nanmin(self._orientations, axis=0)
        result = vec2rowarr(np.array([range_angles[1]]))
        result = add_name(result, self.range_y_angle.__name__)
        return result

    def range_z_angle(self):
        range_angles = np.nanmax(
            self._orientations, axis=0) - np.nanmin(self._orientations, axis=0)
        result = vec2rowarr(np.array([range_angles[2]]))
        result = add_name(result, self.range_z_angle.__name__)
        return result

    def std_angles(self):
        std_angles = np.nanstd(self._orientations, axis=0)
        result = vec2rowarr(np.array(std_angles))
        result = add_name(result, self.std_angles.__name__)
        return result

    def std_x_angle(self):
        std_angles = np.nanstd(self._orientations, axis=0)
        result = vec2rowarr(np.array([std_angles[0]]))
        result = add_name(result, self.std_x_angle.__name__)
        return result

    def std_y_angle(self):
        std_angles = np.nanstd(self._orientations, axis=0)
        result = vec2rowarr(np.array([std_angles[1]]))
        result = add_name(result, self.std_y_angle.__name__)
        return result

    def std_z_angle(self):
        std_angles = np.nanstd(self._orientations, axis=0)
        result = vec2rowarr(np.array([std_angles[2]]))
        result = add_name(result, self.std_z_angle.__name__)
        return result

    def zerocross_x_angle(self):
        ar = self._orientations[:,0]
        asign = np.sign(ar)
        sz = asign == 0
        while sz.any():
            asign[sz] = np.roll(asign, 1)[sz]
            sz = asign == 0
        signchange = ((np.roll(asign, 1) - asign) != 0).astype(np.int32)
        signchange[0] = 0
        if 1 in signchange:
            zcross = 1
        else:
            zcross = 0

        result = vec2rowarr(np.array([zcross]))
        result = add_name(result, self.zerocross_x_angle.__name__)
        return result

    def zerocross_y_angle(self):
        ar = self._orientations[:,1]
        asign = np.sign(ar)
        sz = asign == 0
        while sz.any():
            asign[sz] = np.roll(asign, 1)[sz]
            sz = asign == 0
        signchange = ((np.roll(asign, 1) - asign) != 0).astype(np.int32)
        signchange[0] = 0
        if 1 in signchange:
            zcross = 1
        else:
            zcross = 0

        result = vec2rowarr(np.array([zcross]))
        result = add_name(result, self.zerocross_y_angle.__name__)
        return result

    def zerocross_z_angle(self):
        ar = self._orientations[:,2]
        asign = np.sign(ar)
        sz = asign == 0
        while sz.any():
            asign[sz] = np.roll(asign, 1)[sz]
            sz = asign == 0
        signchange = ((np.roll(asign, 1) - asign) != 0).astype(np.int32)
        signchange[0] = 0
        if 1 in signchange:
            zcross = 1
        else:
            zcross = 0

        result = vec2rowarr(np.array([zcross]))
        result = add_name(result, self.zerocross_z_angle.__name__)
        return result

    def ori_var_sum(self):
        var_angles = np.nanvar(self._orientations, axis=0)
        result = vec2rowarr(np.array([np.sum(var_angles)]))
        result = add_name(result, self.ori_var_sum.__name__)
        return result
    
        
    def ori_range_max(self):
        range_angles = abs(np.nanmax(
            self._orientations, axis=0) - np.nanmin(self._orientations, axis=0))
        result = vec2rowarr(np.array([np.sum(range_angles)]))
        result = add_name(result, self.ori_range_max.__name__)
        return result